"""
channel_dialog.py
GUI to add/edit/remove channels
"""

from sohstationviewer.view.core.db_gui_superclass import UiDBInfoDialog
from sohstationviewer.database.proccessDB import executeDB


class ChannelDialog(UiDBInfoDialog):
    def __init__(self, parent):
        """
        :param parent: QMainWindow/QWidget - the parent widget
        """

        """
        param_choice: [str,] - list of parameters from db table parameters
        """
        self.param_choices = []
        super().__init__(
            parent, ['No.', 'Channel', 'Label', 'Param',
                     'ConvertFactor', 'Unit', 'FixPoint'],
            'channel', 'channels', resize_content_columns=[0, 4, 5, 6],
            need_data_type_choice=True, required_columns={3: 'Param'},
            check_fk=False)
        self.setWindowTitle("Edit/Add/Delete Channels")

    def update_data_table_widget_items(self):
        """
        Create list of parameters to used in widget for selecting parameters
            before update item in self.data_table_widgets
        """
        param_rows = executeDB("SELECT param from parameters")
        self.param_choices = [''] + sorted([d[0] for d in param_rows])
        super(ChannelDialog, self).update_data_table_widget_items()

    def clear_first_row(self):
        """
        Clear content of first row of widgets in self.data_table_widgets in
            data_type that has no channels yet.
        """
        self.data_table_widget.cellWidget(0, 1).setText('')
        self.data_table_widget.cellWidget(0, 2).setText('')
        self.data_table_widget.cellWidget(0, 3).setCurrentIndex(-1)
        self.data_table_widget.cellWidget(0, 4).setText('1')
        self.data_table_widget.cellWidget(0, 5).setText('')
        self.data_table_widget.cellWidget(0, 6).setValue(0)

    def add_row(self, row_idx, fk=False):
        """
        Add a row of widgets to self.data_table_widgets.

        :param row_idx: index of row to add
        :param fk: bool: True if there is a foreign constrain that prevents the
            row to be deleted
        """
        self.add_widget(None, row_idx, 0)  # No.
        self.add_widget(self.data_list, row_idx, 1, foreign_key=fk)  # chanID
        self.add_widget(self.data_list, row_idx, 2)  # label
        self.add_widget(self.data_list, row_idx, 3, choices=self.param_choices)
        self.add_widget(self.data_list, row_idx, 4,
                        field_name='convertFactor')
        self.add_widget(self.data_list, row_idx, 5)  # unit
        self.add_widget(self.data_list, row_idx, 6,
                        range_values=[0, 5])  # fixPoint

    def data_type_changed(self):
        """
        Load new self.data_list when self.data_type_combo_box's value is
            changed
        """
        self.data_type = self.data_type_combo_box.currentText()
        self.update_data_table_widget_items()

    def get_data_list(self):
        """
        Get list of data to fill self.data_table_widgets' content
        """
        channel_rows = executeDB(
            f"SELECT channel, label, param, convertFactor, unit, fixPoint "
            f"FROM Channels "
            f"WHERE dataType='{self.data_type}'")
        return [[d[0], d[1], d[2], d[3],
                 '' if d[4] is None else d[4],
                 d[5]]
                for d in channel_rows]

    def get_row_inputs(self, row_idx):
        """
        Get content from a row of widgets

        :param row_idx: index of row
        """
        return [
            self.data_table_widget.cellWidget(row_idx, 1).text().strip(),
            self.data_table_widget.cellWidget(row_idx, 2).text().strip(),
            self.data_table_widget.cellWidget(row_idx, 3).currentText(),
            float(self.data_table_widget.cellWidget(row_idx, 4).text()),
            self.data_table_widget.cellWidget(row_idx, 5).text(),
            self.data_table_widget.cellWidget(row_idx, 6).value()
        ]

    def remove_row(self, remove_row_idx):
        """
        Remove a row in self.data_table_widget.
        Reset row number of the following rows.

        :param remove_row_idx: index of row to be removed
        """
        self.data_table_widget.remove_row(remove_row_idx)
        for i in range(remove_row_idx, self.data_table_widget.rowCount()):
            cell_widget = self.data_table_widget.cellWidget(i, 0)
            cell_widget.setText(str(i))

    def update_data(self, row, widget_idx, list_idx):
        """
        Prepare insert, update queries and additional condition for common
            delete query then update data of a row from
            self.data_table_widgets' content.

        :param row: list - data of a row
        :param widget_idx: index of row in self.data_table_widgets
        :param list_idx: index of row in self.data_list
        """
        insert_sql = (f"INSERT INTO Channels VALUES"
                      f"('{row[0]}', '{row[1]}', '{row[2]}',"
                      f" {row[3]}, '{row[4]}', {row[5]}, '{self.data_type}')")
        update_sql = (f"UPDATE Channels SET channel='{row[0]}', "
                      f"label='{row[1]}', param='{row[2]}', "
                      f"convertFactor={row[3]}, unit='{row[4]}' "
                      f"fixPoint={row[5]} "
                      f"WHERE channel='%s'"
                      f" AND dataType='{self.data_type}'")
        del_sql_add = f" AND dataType='{self.data_type}'"
        return super().update_data(
            row, widget_idx, list_idx, insert_sql, update_sql, del_sql_add)
